@@ -47,7 +47,6 @@ class MapLineContainer(GenericDiagramWidget):
         self.nodes_list: List[LineLocationGraphicItem] = list()
         self.segments_list: List[MapLineSegment] = list()
         self.enabled = True
-        self.original = True  # TODO: Que es esto?
 
     def set_width_scale(self, branch_scale: float, arrow_scale: float):
         """
@@ -211,6 +210,9 @@ class MapLineContainer(GenericDiagramWidget):
                 connection_elements.append(substation_to_graphics)
                 substation_to_graphics.line_container = self
 
+        br_scale = self.editor.get_branch_width()
+        arrow_scale = self.editor.get_arrow_scale()
+
         # second pass: create the segments
         for i in range(1, len(connection_elements)):
             elm1 = connection_elements[i - 1]
@@ -223,15 +225,19 @@ class MapLineContainer(GenericDiagramWidget):
             elm2.needsUpdate = True
             segment_graphic_object.needsUpdate = True
 
+            segment_graphic_object.set_width(br_scale * segment_graphic_object.width)  # Assign the pen to the line item
+            segment_graphic_object.set_arrow_scale(arrow_scale)
+
             # register the segment in the line
             self.add_segment(segment=segment_graphic_object)
 
             # draw the segment in the scene
             self.editor.add_to_scene(graphic_object=segment_graphic_object)
 
+
         self.update_connectors()
 
-        self.editor.update_device_sizes()
+        # self.editor.update_device_sizes()
 
     def substation_to(self):
         """
@@ -462,7 +468,7 @@ class MapLineContainer(GenericDiagramWidget):
         for line in self.segments_list:
             line.set_enable(val=False)
 
-    def set_arrows_with_power(self, Sf: complex, St: complex) -> None:
+    def set_arrows_with_power(self, Sf: complex | None, St: complex | None) -> None:
         """
 
         :param Sf:
