@@ -15,20 +15,21 @@ from typing import List, Union
 import numpy as np
 import pandas as pd
 from matplotlib import pyplot as plt
+from GridCalEngine.IO.file_system import get_create_gridcal_folder
 
-# GUI importswa
+# GUI imports
 from PySide6 import QtGui, QtWidgets, QtCore
 
 # Engine imports
 from GridCalEngine.Devices.multi_circuit import MultiCircuit
 import GridCalEngine.Simulations as sim
 from GridCalEngine.enumerations import EngineType, DeviceType
-from GridCalEngine.DataStructures.numerical_circuit import NumericalCircuit, compile_numerical_circuit_at
+from GridCalEngine.Compilers.circuit_to_data import compile_numerical_circuit_at
+from GridCalEngine.DataStructures.numerical_circuit import NumericalCircuit
 
 from GridCalEngine.Compilers.circuit_to_bentayga import BENTAYGA_AVAILABLE
 from GridCalEngine.Compilers.circuit_to_newton_pa import NEWTON_PA_AVAILABLE
 from GridCalEngine.Compilers.circuit_to_pgm import PGM_AVAILABLE
-from GridCalEngine.IO.file_system import get_create_gridcal_folder
 import GridCal.Gui.gui_functions as gf
 import GridCal.Session.synchronization_driver as syncdrv
 from GridCal.Gui.AboutDialogue.about_dialogue import AboutDialogueGuiGUI
@@ -36,7 +37,7 @@ from GridCal.Gui.Analysis.AnalysisDialogue import GridAnalysisGUI
 from GridCal.Gui.ContingencyPlanner.contingency_planner_dialogue import ContingencyPlannerGUI
 from GridCal.Gui.CoordinatesInput.coordinates_dialogue import CoordinatesInputGUI
 from GridCal.Gui.general_dialogues import CheckListDialogue, StartEndSelectionDialogue
-from GridCal.Gui.messages import yes_no_question, warning_msg, info_msg, error_msg
+from GridCal.Gui.messages import yes_no_question, warning_msg, info_msg
 from GridCal.Gui.GridGenerator.grid_generator_dialogue import GridGeneratorGUI
 from GridCal.Gui.LoadCatalogue.catalogue_dialogue import CatalogueGUI
 from GridCal.Gui.Main.MainWindow import Ui_mainWindow, QMainWindow
@@ -47,9 +48,8 @@ from GridCal.Session.session import SimulationSession, GcThread
 from GridCal.Gui.SigmaAnalysis.sigma_analysis_dialogue import SigmaAnalysisGUI
 from GridCal.Gui.SyncDialogue.sync_dialogue import SyncDialogueWindow
 from GridCal.Gui.TowerBuilder.LineBuilderDialogue import TowerBuilderGUI
-from GridCal.Gui.general_dialogues import clear_qt_layout
-from GridCal.Gui.console_widget import ConsoleWidget
 from GridCal.Gui.Diagrams.generic_graphics import IS_DARK
+from GridCal.Gui.python_console import PythonConsole
 
 
 def terminate_thread(thread):
@@ -148,10 +148,16 @@ class BaseMainGui(QMainWindow):
         self.find_node_groups_driver: Union[sim.NodeGroupsDriver, None] = None
         self.file_sync_thread = syncdrv.FileSyncThread(self.circuit, None, None)
 
+        self.task_pool = QtCore.QThreadPool()
+
         # simulation start end
         self.simulation_start_index: int = -1
         self.simulation_end_index: int = -1
 
+        # Add console --------------------------------------------------------------------------------------------------
+        self.console = PythonConsole(banner="GridCal Python Console!")
+        self.ui.consoleLayout.addWidget(self.console)
+
         # window pointers ----------------------------------------------------------------------------------------------
         self.file_sync_window: Union[SyncDialogueWindow, None] = None
         self.sigma_dialogue: Union[SigmaAnalysisGUI, None] = None
@@ -195,12 +201,7 @@ class BaseMainGui(QMainWindow):
         # dark mode detection ------------------------------------------------------------------------------------------
         self.ui.dark_mode_checkBox.setChecked(IS_DARK)
 
-        # Console ------------------------------------------------------------------------------------------------------
-        self.console: Union[ConsoleWidget, None] = None
-        try:
-            self.create_console()
-        except TypeError:
-            error_msg('The console has failed because the QtConsole guys have a bug in their package :(')
+
 
         self.calculation_inputs_to_display = None
 
@@ -228,7 +229,7 @@ class BaseMainGui(QMainWindow):
 
         self.ui.grid_name_line_edit.textChanged.connect(self.change_circuit_name)
 
-        # comboboxes
+        # combo-boxes
         self.ui.fromComboBox.currentTextChanged.connect(self.update_from_to_list_views)
         self.ui.toComboBox.currentTextChanged.connect(self.update_from_to_list_views)
 
@@ -249,129 +250,6 @@ class BaseMainGui(QMainWindow):
         if not self.any_thread_running():
             self.LOCK(False)
 
-    def create_console(self) -> None:
-        """
-        Create console
-        """
-        if self.console is not None:
-            clear_qt_layout(self.ui.pythonConsoleTab.layout())
-
-        self.console = ConsoleWidget(customBanner="GridCal console.\n\n"
-                                                  "type hlp() to see the available specific commands.\n\n"
-                                                  "the following libraries are already loaded:\n"
-                                                  "np: numpy\n"
-                                                  "pd: pandas\n"
-                                                  "plt: matplotlib\n"
-                                                  "app: This instance of GridCal\n"
-                                                  "circuit: The current grid\n"
-                                                  "user_folder: path to the user folder\n")
-
-        self.console.buffer_size = 10000
-
-        # add the console widget to the user interface
-        self.ui.pythonConsoleTab.layout().addWidget(self.console)
-
-        # push some variables to the console
-        self.console.push_vars(
-            {"hlp": self.print_console_help,
-             "np": np,
-             "pd": pd,
-             "plt": plt,
-             "clc": self.clc,
-             'app': self,
-             'circuit': self.circuit,
-             'user_folder': get_create_gridcal_folder,
-             }
-        )
-
-        if IS_DARK:
-            self.console.set_dark_theme()
-        else:
-            self.console.set_light_theme()
-
-    @staticmethod
-    def print_console_help():
-        """
-        Print the console help in the console
-        @return:
-        """
-        print('GridCal internal commands.\n')
-        print('If a command is unavailable is because the study has not been executed yet.')
-
-        print('\n\nclc():\tclear the console.')
-
-        print('\n\nApp functions:')
-        print('\tapp.new_project(): Clear all.')
-        print('\tapp.open_file(): Prompt to load GridCal compatible file')
-        print('\tapp.save_file(): Prompt to save GridCal file')
-        print('\tapp.export_diagram(): Prompt to export the diagram in png.')
-        print('\tapp.create_schematic_from_api(): Create the schematic from the circuit information.')
-        print('\tapp.adjust_all_node_width(): Adjust the width of all the nodes according to their name.')
-        print('\tapp.numerical_circuit: get compilation of the assets.')
-        print('\tapp.islands: get compilation of the assets split into the topological islands.')
-
-        print('\n\nCircuit functions:')
-        print('\tapp.circuit.plot_graph(): Plot a graph in a Matplotlib window. Call plt.show() after.')
-
-        print('\n\nPower flow results:')
-        print('\tapp.session.power_flow.voltage:\t the nodal voltages in per unit')
-        print('\tapp.session.power_flow.current:\t the branch currents in per unit')
-        print('\tapp.session.power_flow.loading:\t the branch loading in %')
-        print('\tapp.session.power_flow.losses:\t the branch losses in per unit')
-        print('\tapp.session.power_flow.power:\t the nodal power Injections in per unit')
-        print('\tapp.session.power_flow.Sf:\t the branch power Injections in per unit at the "from" side')
-        print('\tapp.session.power_flow.St:\t the branch power Injections in per unit at the "to" side')
-
-        print('\n\nShort circuit results:')
-        print('\tapp.session.short_circuit.voltage:\t the nodal voltages in per unit')
-        print('\tapp.session.short_circuit.current:\t the branch currents in per unit')
-        print('\tapp.session.short_circuit.loading:\t the branch loading in %')
-        print('\tapp.session.short_circuit.losses:\t the branch losses in per unit')
-        print('\tapp.session.short_circuit.power:\t the nodal power Injections in per unit')
-        print('\tapp.session.short_circuit.power_from:\t the branch power Injections in per unit at the "from" side')
-        print('\tapp.session.short_circuit.power_to:\t the branch power Injections in per unit at the "to" side')
-        print('\tapp.session.short_circuit.short_circuit_power:\t Short circuit power in MVA of the grid nodes')
-
-        print('\n\nOptimal power flow results:')
-        print('\tapp.session.optimal_power_flow.voltage:\t the nodal voltages angles in rad')
-        print('\tapp.session.optimal_power_flow.load_shedding:\t the branch loading in %')
-        print('\tapp.session.optimal_power_flow.losses:\t the branch losses in per unit')
-        print('\tapp.session.optimal_power_flow.Sbus:\t the nodal power Injections in MW')
-        print('\tapp.session.optimal_power_flow.Sf:\t the branch power Sf')
-
-        print('\n\nTime series power flow results:')
-        print('\tapp.session.power_flow_ts.time:\t Profiles time index (pandas DateTimeIndex object)')
-        print('\tapp.session.power_flow_ts.load_profiles:\t Load profiles matrix (row: time, col: node)')
-        print('\tapp.session.power_flow_ts.gen_profiles:\t Generation profiles matrix (row: time, col: node)')
-        print('\tapp.session.power_flow_ts.voltages:\t nodal voltages results matrix (row: time, col: node)')
-        print('\tapp.session.power_flow_ts.currents:\t Branches currents results matrix (row: time, col: branch)')
-        print('\tapp.session.power_flow_ts.loadings:\t Branches loadings results matrix (row: time, col: branch)')
-        print('\tapp.session.power_flow_ts.losses:\t Branches losses results matrix (row: time, col: branch)')
-
-        print('\n\nVoltage stability power flow results:')
-        print('\tapp.session.continuation_power_flow.voltage:\t Voltage values for every power multiplication factor.')
-        print('\tapp.session.continuation_power_flow.lambda:\t Value of power multiplication factor applied')
-        print('\tapp.session.continuation_power_flow.Sf:\t Power values for every power multiplication factor.')
-
-        print('\n\nMonte Carlo power flow results:')
-        print('\tapp.session.stochastic_power_flow.V_avg:\t nodal voltage average result.')
-        print('\tapp.session.stochastic_power_flow.I_avg:\t branch current average result.')
-        print('\tapp.session.stochastic_power_flow.Loading_avg:\t branch loading average result.')
-        print('\tapp.session.stochastic_power_flow.Losses_avg:\t branch losses average result.')
-        print('\tapp.session.stochastic_power_flow.V_std:\t nodal voltage standard deviation result.')
-        print('\tapp.session.stochastic_power_flow.I_std:\t branch current standard deviation result.')
-        print('\tapp.session.stochastic_power_flow.Loading_std:\t branch loading standard deviation result.')
-        print('\tapp.session.stochastic_power_flow.Losses_std:\t branch losses standard deviation result.')
-        print('\tapp.session.stochastic_power_flow.V_avg_series:\t nodal voltage average series.')
-        print('\tapp.session.stochastic_power_flow.V_std_series:\t branch current standard deviation series.')
-        print('\tapp.session.stochastic_power_flow.error_series:\t Monte Carlo error series (the convergence value).')
-        print('The same for app.latin_hypercube_sampling')
-
-    def clc(self):
-        """
-        Clear the console
-        """
-        self.console.clear()
 
     @property
     def file_name(self) -> str:
@@ -796,3 +674,106 @@ class BaseMainGui(QMainWindow):
                 self.circuit.contingencies += self.contingency_planner_dialogue.contingencies
             else:
                 info_msg(text="No contingencies were generated :/", title="Contingency planner")
+
+    def clear_console(self):
+        """
+        Clear console output
+        """
+        self.console.clear()
+
+    def reset_console(self):
+        """
+        Reset console
+        :return:
+        """
+        self.console.reset()
+        self.add_console_vars()
+
+    def print_console_help(self):
+
+        self.console.append_output("")
+        self.console.append_output('GridCal internal commands.\n')
+        self.console.append_output('If a command is unavailable is because the study has not been executed yet.')
+
+        self.console.append_output('\n\nclc():\tclear the console.')
+
+        self.console.append_output('\n\nApp functions:')
+        self.console.append_output('\tapp.new_project(): Clear all.')
+        self.console.append_output('\tapp.open_file(): Prompt to load GridCal compatible file')
+        self.console.append_output('\tapp.save_file(): Prompt to save GridCal file')
+        self.console.append_output('\tapp.export_diagram(): Prompt to export the diagram in png.')
+        self.console.append_output('\tapp.create_schematic_from_api(): Create the schematic from the circuit information.')
+        self.console.append_output('\tapp.adjust_all_node_width(): Adjust the width of all the nodes according to their name.')
+        self.console.append_output('\tapp.numerical_circuit: get compilation of the assets.')
+        self.console.append_output('\tapp.islands: get compilation of the assets split into the topological islands.')
+
+        self.console.append_output('\n\nCircuit functions:')
+        self.console.append_output('\tapp.circuit.plot_graph(): Plot a graph in a Matplotlib window. Call plt.show() after.')
+
+        self.console.append_output('\n\nPower flow results:')
+        self.console.append_output('\tapp.session.power_flow.voltage:\t the nodal voltages in per unit')
+        self.console.append_output('\tapp.session.power_flow.current:\t the branch currents in per unit')
+        self.console.append_output('\tapp.session.power_flow.loading:\t the branch loading in %')
+        self.console.append_output('\tapp.session.power_flow.losses:\t the branch losses in per unit')
+        self.console.append_output('\tapp.session.power_flow.power:\t the nodal power Injections in per unit')
+        self.console.append_output('\tapp.session.power_flow.Sf:\t the branch power Injections in per unit at the "from" side')
+        self.console.append_output('\tapp.session.power_flow.St:\t the branch power Injections in per unit at the "to" side')
+
+        self.console.append_output('\n\nShort circuit results:')
+        self.console.append_output('\tapp.session.short_circuit.voltage:\t the nodal voltages in per unit')
+        self.console.append_output('\tapp.session.short_circuit.current:\t the branch currents in per unit')
+        self.console.append_output('\tapp.session.short_circuit.loading:\t the branch loading in %')
+        self.console.append_output('\tapp.session.short_circuit.losses:\t the branch losses in per unit')
+        self.console.append_output('\tapp.session.short_circuit.power:\t the nodal power Injections in per unit')
+        self.console.append_output('\tapp.session.short_circuit.power_from:\t the branch power Injections in per unit at the "from" side')
+        self.console.append_output('\tapp.session.short_circuit.power_to:\t the branch power Injections in per unit at the "to" side')
+        self.console.append_output('\tapp.session.short_circuit.short_circuit_power:\t Short circuit power in MVA of the grid nodes')
+
+        self.console.append_output('\n\nOptimal power flow results:')
+        self.console.append_output('\tapp.session.optimal_power_flow.voltage:\t the nodal voltages angles in rad')
+        self.console.append_output('\tapp.session.optimal_power_flow.load_shedding:\t the branch loading in %')
+        self.console.append_output('\tapp.session.optimal_power_flow.losses:\t the branch losses in per unit')
+        self.console.append_output('\tapp.session.optimal_power_flow.Sbus:\t the nodal power Injections in MW')
+        self.console.append_output('\tapp.session.optimal_power_flow.Sf:\t the branch power Sf')
+
+        self.console.append_output('\n\nTime series power flow results:')
+        self.console.append_output('\tapp.session.power_flow_ts.time:\t Profiles time index (pandas DateTimeIndex object)')
+        self.console.append_output('\tapp.session.power_flow_ts.load_profiles:\t Load profiles matrix (row: time, col: node)')
+        self.console.append_output('\tapp.session.power_flow_ts.gen_profiles:\t Generation profiles matrix (row: time, col: node)')
+        self.console.append_output('\tapp.session.power_flow_ts.voltages:\t nodal voltages results matrix (row: time, col: node)')
+        self.console.append_output('\tapp.session.power_flow_ts.currents:\t Branches currents results matrix (row: time, col: branch)')
+        self.console.append_output('\tapp.session.power_flow_ts.loadings:\t Branches loadings results matrix (row: time, col: branch)')
+        self.console.append_output('\tapp.session.power_flow_ts.losses:\t Branches losses results matrix (row: time, col: branch)')
+
+        self.console.append_output('\n\nVoltage stability power flow results:')
+        self.console.append_output('\tapp.session.continuation_power_flow.voltage:\t Voltage values for every power multiplication factor.')
+        self.console.append_output('\tapp.session.continuation_power_flow.lambda:\t Value of power multiplication factor applied')
+        self.console.append_output('\tapp.session.continuation_power_flow.Sf:\t Power values for every power multiplication factor.')
+
+        self.console.append_output('\n\nMonte Carlo power flow results:')
+        self.console.append_output('\tapp.session.stochastic_power_flow.V_avg:\t nodal voltage average result.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.I_avg:\t branch current average result.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.Loading_avg:\t branch loading average result.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.Losses_avg:\t branch losses average result.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.V_std:\t nodal voltage standard deviation result.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.I_std:\t branch current standard deviation result.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.Loading_std:\t branch loading standard deviation result.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.Losses_std:\t branch losses standard deviation result.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.V_avg_series:\t nodal voltage average series.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.V_std_series:\t branch current standard deviation series.')
+        self.console.append_output('\tapp.session.stochastic_power_flow.error_series:\t Monte Carlo error series (the convergence value).')
+        self.console.append_output('The same for app.latin_hypercube_sampling')
+
+    def add_console_vars(self):
+        """
+        Add vars to the console
+        :return:
+        """
+        self.console.add_var("hlp", self.print_console_help)
+        self.console.add_var("np", np)
+        self.console.add_var("pd", pd)
+        self.console.add_var("plt", plt)
+        self.console.add_var("clc", self.clear_console)
+        self.console.add_var('app', self)
+        self.console.add_var('circuit', self.circuit)
+        self.console.add_var('user_folder', get_create_gridcal_folder)
\ No newline at end of file
