@@ -4,6 +4,7 @@
 # SPDX-License-Identifier: MPL-2.0
 
 import os
+import asyncio
 from typing import Union, List, Callable
 import pandas as pd
 from PySide6 import QtWidgets
@@ -23,7 +24,7 @@ from GridCal.Gui.Main.SubClasses.Settings.configuration import ConfigurationMain
 from GridCalEngine.Devices.multi_circuit import MultiCircuit
 from GridCalEngine.Compilers.circuit_to_newton_pa import NEWTON_PA_AVAILABLE
 from GridCalEngine.Compilers.circuit_to_pgm import PGM_AVAILABLE
-from GridCalEngine.DataStructures.numerical_circuit import compile_numerical_circuit_at
+from GridCalEngine.Compilers.circuit_to_data import compile_numerical_circuit_at
 from GridCalEngine.enumerations import CGMESVersions, SimulationTypes
 from GridCalEngine.basic_structures import Logger
 from GridCalEngine.IO.gridcal.contingency_parser import import_contingencies_from_json, export_contingencies_json_file
@@ -51,8 +52,8 @@ class IoMain(ConfigurationMain):
         self.rosetta_gui: Union[RosetaExplorerGUI, None] = None
 
         self.accepted_extensions = ['.gridcal', '.dgridcal', '.xlsx', '.xls', '.sqlite', '.gch5',
-                                    '.dgs', '.m', '.raw', '.RAW', '.json',
-                                    '.ejson2', '.ejson3',
+                                    '.dgs', '.m', '.raw', '.RAW', '.json', '.uct',
+                                    '.ejson2', '.ejson3', '.p', '.nc', '.hdf5',
                                     '.xml', '.rawx', '.zip', '.dpx', '.epc', '.EPC',
                                     '.gcplugin']
 
@@ -76,7 +77,7 @@ class IoMain(ConfigurationMain):
 
         self.ui.actionNew_project.triggered.connect(self.new_project)
         self.ui.actionOpen_file.triggered.connect(self.open_file)
-        self.ui.actionAdd_circuit.triggered.connect(self.add_circuit)
+        self.ui.actionAdd_circuit.triggered.connect(self.import_circuit)
         self.ui.actionExport_circuit_differential.triggered.connect(self.export_circuit_differential)
         self.ui.actionSave.triggered.connect(self.save_file)
         self.ui.actionSave_as.triggered.connect(self.save_file_as)
@@ -152,7 +153,7 @@ class IoMain(ConfigurationMain):
                     if any_grid_delta and not any_normal_grid:
                         # only grid deltas...
                         self.open_file_now(filenames=file_names,
-                                           post_function=self.post_add_circuit)
+                                           post_function=self.post_import_circuit)
                     else:
                         quit_msg = ("Are you sure that you want to quit the current grid and open a new one?"
                                     "\n If the process is cancelled the grid will remain.")
@@ -198,7 +199,6 @@ class IoMain(ConfigurationMain):
 
         self.clear_stuff_running()
         self.clear_results()
-        self.create_console()
 
         if create_default_diagrams:
             self.add_complete_bus_branch_diagram()
@@ -261,8 +261,8 @@ class IoMain(ConfigurationMain):
             files_types += "*.dgridcal "
 
         files_types += "*.gch5 *.xlsx *.xls *.sqlite *.dgs "
-        files_types += "*.m *.raw *.RAW *.rawx *.json *.ejson2 *.ejson3 *.xml "
-        files_types += "*.zip *.dpx *.epc *.EPC *.nc *.hdf5"
+        files_types += "*.m *.raw *.RAW *.rawx *.uct *.json *.ejson2 *.ejson3 *.xml "
+        files_types += "*.zip *.dpx *.epc *.EPC *.nc *.hdf5 *.p"
 
         dialogue = QtWidgets.QFileDialog(None,
                                          caption=title,
@@ -343,12 +343,10 @@ class IoMain(ConfigurationMain):
 
                 if self.circuit.has_diagrams():
                     # create the diagrams that came with the file
+                    # task = gf.AsyncTask(self.create_circuit_stored_diagrams)
+                    # self.task_pool.start(task)
                     self.create_circuit_stored_diagrams()
 
-                    if len(self.diagram_widgets_list) > 0:
-                        diagram = self.diagram_widgets_list[0]
-                        self.set_diagram_widget(diagram)
-
                 else:
                     if self.circuit.get_bus_number() > 300:
                         quit_msg = ("The grid is quite large, hence the schematic might be slow.\n"
@@ -409,14 +407,14 @@ class IoMain(ConfigurationMain):
                     # else, show the logger if it is necessary
                     if len(self.open_file_thread_object.logger) > 0:
                         dlg = LogsDialogue('Open file logger', self.open_file_thread_object.logger)
-                        dlg.exec_()
+                        dlg.exec()
 
             else:
                 warning_msg(text='Error while loading the file(s)')
                 # else, show the logger if it is necessary
                 if len(self.open_file_thread_object.logger) > 0:
                     dlg = LogsDialogue('Open file logger', self.open_file_thread_object.logger)
-                    dlg.exec_()
+                    dlg.exec()
         else:
             # center nodes
             diagram = self.get_selected_diagram_widget()
@@ -450,10 +448,10 @@ class IoMain(ConfigurationMain):
                         found = True
 
                 if found:
-                    ok = yes_no_question( f"There is a plugin already: "
-                                    f"{plugin.name} {plugin.version} "
-                                    f"The new plugin is {info.version}. "
-                                    f"Install?", "Plugin install")
+                    ok = yes_no_question(f"There is a plugin already: "
+                                         f"{plugin.name} {plugin.version} "
+                                         f"The new plugin is {info.version}. "
+                                         f"Install?", "Plugin install")
                     if not ok:
                         return
 
@@ -482,14 +480,14 @@ class IoMain(ConfigurationMain):
         else:
             return None
 
-    def add_circuit(self):
+    def import_circuit(self):
         """
         Prompt to add another circuit
         """
-        self.open_file_threaded(post_function=self.post_add_circuit,
+        self.open_file_threaded(post_function=self.post_import_circuit,
                                 allow_diff_file_format=True)
 
-    def post_add_circuit(self):
+    def post_import_circuit(self):
         """
         Stuff to do after opening another circuit
         :return: Nothing
@@ -503,58 +501,77 @@ class IoMain(ConfigurationMain):
             if len(self.open_file_thread_object.logger) > 0:
                 dlg = LogsDialogue('Open file logger',
                                    self.open_file_thread_object.logger)
-                dlg.exec_()
+                dlg.exec()
 
             if self.open_file_thread_object.valid:
 
-                logger = self.circuit.add_circuit(new_circuit)
-
-                if len(logger) > 0:
-                    dlg = LogsDialogue('File merge logger', logger)
-                    dlg.exec_()
-
                 dlg2 = CustomQuestionDialogue(title="Grid differential",
-                                              question="How do you want to represent the loaded grid?",
-                                              answer1="Create new diagram",
-                                              answer2="Add to current diagram")
-                dlg2.exec_()
+                                              question="How do you want to proceed?",
+                                              answer1="Add grid",
+                                              answer2="Merge grid")
+                dlg2.exec()
 
                 if dlg2.accepted_answer == 1:
                     # Create a blank diagram and add to it
-                    diagram_widget = self.create_blank_schematic_diagram(name=new_circuit.name)
+                    logger = self.circuit.add_circuit(new_circuit)
+
+                    # for sure, we want to add to the current schematic
+                    diagram_widget = self.get_selected_diagram_widget()
 
                 elif dlg2.accepted_answer == 2:
-                    diagram_widget = self.get_selected_diagram_widget()
+                    logger = self.circuit.merge_circuit(new_circuit)
+
+                    if len(logger) > 0:
+                        dlg = LogsDialogue('File merge logger', logger)
+                        dlg.exec()
+
+                    dlg3 = CustomQuestionDialogue(title="Grid differential",
+                                                  question="How do you want to represent the loaded grid?",
+                                                  answer1="Create new diagram",
+                                                  answer2="Add to current diagram")
+                    dlg3.exec()
+
+                    if dlg3.accepted_answer == 1:
+                        # Create a blank diagram and add to it
+                        diagram_widget = self.create_blank_schematic_diagram(name=new_circuit.name)
+
+                    elif dlg3.accepted_answer == 2:
+                        diagram_widget = self.get_selected_diagram_widget()
+
+                    else:
+                        # not important
+                        diagram_widget = None
 
                 else:
                     return
 
-                if isinstance(diagram_widget, SchematicWidget):
-                    injections_by_bus = new_circuit.get_injection_devices_grouped_by_bus()
-                    injections_by_fluid_node = new_circuit.get_injection_devices_grouped_by_fluid_node()
-                    injections_by_cn = new_circuit.get_injection_devices_grouped_by_cn()
-                    diagram_widget.add_elements_to_schematic(buses=new_circuit.buses,
-                                                             connectivity_nodes=new_circuit.connectivity_nodes,
-                                                             busbars=new_circuit.bus_bars,
-                                                             lines=new_circuit.lines,
-                                                             dc_lines=new_circuit.dc_lines,
-                                                             transformers2w=new_circuit.transformers2w,
-                                                             transformers3w=new_circuit.transformers3w,
-                                                             hvdc_lines=new_circuit.hvdc_lines,
-                                                             vsc_devices=new_circuit.vsc_devices,
-                                                             upfc_devices=new_circuit.upfc_devices,
-                                                             switches=new_circuit.switch_devices,
-                                                             fluid_nodes=new_circuit.fluid_nodes,
-                                                             fluid_paths=new_circuit.fluid_paths,
-                                                             injections_by_bus=injections_by_bus,
-                                                             injections_by_fluid_node=injections_by_fluid_node,
-                                                             injections_by_cn=injections_by_cn,
-                                                             explode_factor=1.0,
-                                                             prog_func=None,
-                                                             text_func=None)
-                    diagram_widget.set_selected_buses(buses=new_circuit.buses)
-                else:
-                    info_msg("No schematic diagram was selected...", title="Add to current diagram")
+                if diagram_widget is not None:
+                    if isinstance(diagram_widget, SchematicWidget):
+                        injections_by_bus = new_circuit.get_injection_devices_grouped_by_bus()
+                        injections_by_fluid_node = new_circuit.get_injection_devices_grouped_by_fluid_node()
+                        injections_by_cn = new_circuit.get_injection_devices_grouped_by_cn()
+                        diagram_widget.add_elements_to_schematic(buses=new_circuit.buses,
+                                                                 connectivity_nodes=new_circuit.connectivity_nodes,
+                                                                 busbars=new_circuit.bus_bars,
+                                                                 lines=new_circuit.lines,
+                                                                 dc_lines=new_circuit.dc_lines,
+                                                                 transformers2w=new_circuit.transformers2w,
+                                                                 transformers3w=new_circuit.transformers3w,
+                                                                 hvdc_lines=new_circuit.hvdc_lines,
+                                                                 vsc_devices=new_circuit.vsc_devices,
+                                                                 upfc_devices=new_circuit.upfc_devices,
+                                                                 switches=new_circuit.switch_devices,
+                                                                 fluid_nodes=new_circuit.fluid_nodes,
+                                                                 fluid_paths=new_circuit.fluid_paths,
+                                                                 injections_by_bus=injections_by_bus,
+                                                                 injections_by_fluid_node=injections_by_fluid_node,
+                                                                 injections_by_cn=injections_by_cn,
+                                                                 explode_factor=1.0,
+                                                                 prog_func=None,
+                                                                 text_func=None)
+                        diagram_widget.set_selected_buses(buses=new_circuit.buses)
+                    else:
+                        info_msg("No schematic diagram was selected...", title="Add to current diagram")
 
     def export_circuit_differential(self):
         """
@@ -570,7 +587,7 @@ class IoMain(ConfigurationMain):
                                     title="Load base grid to compare...")
         else:
             dlg = LogsDialogue('This circuit has duplicated idtags :(', logger)
-            dlg.exec_()
+            dlg.exec()
 
     def post_create_circuit_differential(self):
         """
@@ -583,7 +600,7 @@ class IoMain(ConfigurationMain):
 
             if self.open_file_thread_object.logger.has_logs():
                 dlg = LogsDialogue('Open file logger', self.open_file_thread_object.logger)
-                dlg.exec_()
+                dlg.exec()
 
             if self.open_file_thread_object.valid:
 
@@ -604,7 +621,7 @@ class IoMain(ConfigurationMain):
 
                         if diff_logger.has_logs():
                             dlg = LogsDialogue('Grid differences', diff_logger)
-                            dlg.exec_()
+                            dlg.exec()
 
                         # select the file to save
                         filename, type_selected = QtWidgets.QFileDialog.getSaveFileName(self, 'Save file',
@@ -806,7 +823,7 @@ class IoMain(ConfigurationMain):
         if self.save_file_thread_object.logger is not None:
             if len(self.save_file_thread_object.logger) > 0:
                 dlg = LogsDialogue('Save file logger', self.save_file_thread_object.logger)
-                dlg.exec_()
+                dlg.exec()
 
         self.stuff_running_now.remove('file_save')
 
@@ -826,7 +843,7 @@ class IoMain(ConfigurationMain):
         """
         self.grid_generator_dialogue = GridGeneratorGUI(parent=self)
         self.grid_generator_dialogue.resize(int(1.61 * 600.0), 550)  # golden ratio
-        self.grid_generator_dialogue.exec_()
+        self.grid_generator_dialogue.exec()
 
         if self.grid_generator_dialogue.applied:
 
@@ -872,7 +889,7 @@ class IoMain(ConfigurationMain):
         :return:
         """
         self.coordinates_window = CoordinatesInputGUI(self, self.circuit.get_buses())
-        self.coordinates_window.exec_()
+        self.coordinates_window.exec()
         self.set_xy_from_lat_lon()
 
     def export_object_profiles(self):
@@ -938,7 +955,7 @@ class IoMain(ConfigurationMain):
         if self.export_all_thread_object is not None:
             if self.export_all_thread_object.logger.has_logs():
                 dlg = LogsDialogue('Export all', self.export_all_thread_object.logger)
-                dlg.exec_()
+                dlg.exec()
 
         if len(self.stuff_running_now) == 0:
             self.UNLOCK()
@@ -1027,7 +1044,7 @@ class IoMain(ConfigurationMain):
 
             if len(logger) > 0:
                 dlg = LogsDialogue('Contingencies import', logger)
-                dlg.exec_()
+                dlg.exec()
 
     def export_contingencies(self):
         """
@@ -1127,4 +1144,3 @@ class IoMain(ConfigurationMain):
         self.setup_time_sliders()
         self.get_circuit_snapshot_datetime()
         self.change_theme_mode()
-
