@@ -42,4 +42,4 @@ if __name__ == "__main__":
     app = QtWidgets.QApplication(sys.argv)
     window = ExcelDialog()
     window.show()
-    sys.exit(app.exec_())
\ No newline at end of file
+    sys.exit(app.exec())
\ No newline at end of file
